-- Copyright (C) 2010 Noé Falzon

-- This program is  free software:  you can redistribute it and/or modify it
-- under  the terms  of the  GNU General Public License  as published by the
-- Free Software Foundation, version 3 of the License.
--
-- This program  is  distributed  in the hope  that it will  be useful,  but
-- WITHOUT  ANY WARRANTY;  without  even the implied  warranty of MERCHANTA-
-- BILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the  GNU General Public License  along
-- with this program. If not, see <http://www.gnu.org/licenses/>.

require "class"

Dude = class()


function Dude:init(game,team,x,y)

	self.x = 0
	self.y = 0
	self.z = 0
	
	self.x = x or MR()*game.size
	self.y = y or MR()*game.size
	
	self.gx = self.x
	self.gy = self.y
	
	
	self.z = game:altitude(self.x,self.y)
	if self.z <= game.sealevel then
		return
	end

	if not dudemesh then
		dudemesh = obj.load("meshes/dude.obj")
		obj.makeDisplayList(dudemesh, false, false)
	end
	
	return self
end

function Dude:update(dt)
	
	
	if self.truetarget then
	
		-- check if there's a boat
		for i,v in ipairs(game.boats) do
			if dist2(self,v) < 1 then
				local success = v:embark()
				
				if success then
					self.die = true
				else
					self.truetarget = false
					self.gx = self.x
					self.gy = self.y
				end
				
				break
			end
		end
	
		-- check if there's a tele tower
		for i,v in ipairs(game.teles) do
			if dist2(self,v) < 1 then
				v:teleport(self)
				
				self.truetarget = false
				self.gx = self.x
				self.gy = self.y
				break
			end
		end
		
		-- or even a shelter
		for i,v in ipairs(game.shelters) do
			if dist2(self,v) < 1 then
				local success = v:embark()
				
				if success then
					self.die = true
				else
					self.truetarget = false
					self.gx = self.x
					self.gy = self.y
				end
				
				break
			end
		end
		
	end
	
	if not self.truetarget then
		-- run from sea
		
		if self.z - game.sealevel < 0.1 then
		
			local dx,dy = game.land:gradient(self.x,self.y)
			self.gx = self.x + dx*2
			self.gy = self.y + dy*2
		end
		
		-- sometimes be random
		if MR()<0.5*dt then
			self.gx = self.x + MR2()*2
			self.gy = self.y + MR2()*2
		end
		
		-- sometimes remember the beacons
		if MR()<0.5*dt then
			self:checkBeacons()
		end
	end	
	
	-- point to goal
	
	local dx = self.gx - self.x
	local dy = self.gy - self.y
	local dist = math.sqrt( dx^2 + dy^2 )
	
	if dist < 0.1 then
		dx,dy = 0,0
		dist = 1
	end
	
	
	-- move
	local speed = 0.5

	
	local x2 = self.x + dx/dist * speed * dt
	local y2 = self.y + dy/dist * speed * dt
	
	-- dont fall off
	
	if x2 > game.size-0.1 then x2 = game.size-0.1 end
	if y2 > game.size-0.1 then y2 = game.size-0.1 end
	if x2 < 0.1 then x2 = 0.1 end
	if y2 < 0.1 then y2 = 0.1 end
	
	-- don't run in the sea
	if game:altitude(x2,y2) <= game.sealevel then
		
	else
		self.x = x2
		self.y = y2
	end
	
	self:updateAltitude()
end

function Dude:updateAltitude()
		
	self.z = game:altitude(self.x,self.y)
	
	if self.z <= game.sealevel then
		self.die = true
	end
end

function Dude:checkBeacons()
	
	local mind = math.huge
	local beac
	
	for i,v in ipairs(game.beacons) do
		local d = (v.x - self.x)^2 + (v.y - self.y)^2
		if d < mind  and d < v.rad^2 then
			mind = d
			beac = v
		end
	end
	
	if beac then
		self.gx = beac.x
		self.gy = beac.y
	else
		self.gx = self.x
		self.gy = self.y
	end
end

function Dude:draw()
	
	
	if (game.camx-self.x)^2 + (game.camy-self.y)^2 + (game.camz)^2< 15^2 then
		gl.PushMatrix()
			
		gl.Enable('LIGHTING')
		gl.Translate(self.x,self.y,self.z+0.1)
		gl.Scale(0.2,0.2,0.2)
		obj.drawDisplayList(dudemesh)
		gl.Disable('LIGHTING')
		gl.PopMatrix()
	else
		return {self.x,self.y,self.z+0.1}
	end
	
end

function Dude:target(x,y)
	self.gx = x
	self.gy = y
	
	self.truetarget = true
end
