-- Copyright (C) 2010 Noé Falzon

-- This program is  free software:  you can redistribute it and/or modify it
-- under  the terms  of the  GNU General Public License  as published by the
-- Free Software Foundation, version 3 of the License.
--
-- This program  is  distributed  in the hope  that it will  be useful,  but
-- WITHOUT  ANY WARRANTY;  without  even the implied  warranty of MERCHANTA-
-- BILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the  GNU General Public License  along
-- with this program. If not, see <http://www.gnu.org/licenses/>.

require "class"
require "keymap"

require "Dude"
require "Beacon"
require "Boat"
require "Menu"
require "Tele"
require "Shelter"

Game = class()

local maxcamz = 100

col = {
	water = C{141,26,173},
	land1 = C{233,142,23},
	land2 = {0,1,0},
	dudes = C{131,191,10},
	buildings = C{194,36,0}, 
	hot = C{255,125,0},
	menu = {0.8,0.8,0.8},
}

landcol = 1

function Game:init(n)
	
	self.land = Heightfield(n)
	self.size = n
	
	self.sealevel = 0
	
	self.camx = n/2
	self.camy = n/2
	self.camalt = self:altitude(self.camx,self.camy)
	self.camz = 30
	self.camdir = 0
	
	self.camx2 = 0
	self.camy2 = 0
	self.camalt2 = self.camalt
	
	
	-- light
	self.light = 0
	
	gl.Light('LIGHT0','DIFFUSE',{1,1,1})
	gl.Light('LIGHT0','SPECULAR',{1,1,1})
	gl.Light('LIGHT0','AMBIENT',{0.1,0.1,0.1})
	gl.LightModel('LIGHT_MODEL_AMBIENT',{0.1,0.1,0.1})

	gl.Enable('LIGHTING')
	gl.Enable('LIGHT0')
	
	-- drawing param
	wireframe = false
	
	-- make some dudes
	
	self.dudes = {}
	while #self.dudes < 500 do
		local dud = Dude(self,1)
		if dud then
			TI(self.dudes, dud)
		end
	end
	
	-- other stuff
	self.beacons = {}
	self.boats = {}
	self.teles = {}
	self.shelters = {}
	
	-- force load some meshes
	if not telemesh then
		telemesh = obj.load("meshes/tele.obj")
		obj.makeDisplayList(telemesh, false, false)
	end
	
	if not deletemesh then
		deletemesh = obj.load("meshes/delete.obj")
		obj.makeDisplayList(deletemesh, false, false)
	end
	
	if not beaconmesh then
		beaconmesh = obj.load("meshes/beacon.obj")
		obj.makeDisplayList(beaconmesh, false, false)
	end
	
	if not sheltermesh then
		sheltermesh = obj.load("meshes/shelter.obj")
		obj.makeDisplayList(sheltermesh, false, false)
	end
	
	if not boatmesh then
		boatmesh = obj.load("meshes/boat.obj")
		obj.makeDisplayList(boatmesh, false, false)
	end

	
	return self
end

function Game:draw()
	
	--gl.ClearColor(0,0,0.7,0)
	gl.Clear('COLOR_BUFFER_BIT,DEPTH_BUFFER_BIT') -- Clear Screen And Depth Buffer
	
	gl.MatrixMode('MODELVIEW')
	gl.LoadIdentity()             -- Reset The Current Modelview Matrix
	
	gl.PushMatrix()
	
	
	gl.Enable('LIGHTING')

	-- Place camera
	
	
	gl.Translate(0,0,-self.camz)
	gl.Rotate(-lerp(80,30,self.camz/maxcamz),1,0,0)
	gl.Rotate(self.camdir,0,0,1)
	gl.Translate(-self.camx2,-self.camy2,-self.camalt2)
	
	
	-- Place light
	gl.PushMatrix()
		
		gl.Rotate(-30,1,0,0)
		gl.Rotate(-self.light,0,1,0)
			
		gl.Light('LIGHT0','POSITION',{0,0,1,0})

	gl.PopMatrix()	
	
	-- Terrain

	gl.ClipPlane('CLIP_PLANE0',{0,0,1,-self.sealevel})
	gl.Enable('CLIP_PLANE0')
	self.land:draw()
	gl.Disable('CLIP_PLANE0')

	
	-- Water

	gl.Color(col.water)
	gl.Begin('QUADS')
		gl.Vertex(0,0,self.sealevel)
		gl.Vertex(0,self.size,self.sealevel)
		gl.Vertex(self.size,self.size,self.sealevel)
		gl.Vertex(self.size,0,self.sealevel)
	gl.End('QUADS')

		
	gl.Disable('LIGHTING')

	-- dudes
	gl.Color(col.dudes)	
	
	local dots = {}
		
	for i,v in ipairs(self.dudes) do
		local ret = v:draw()
		if ret then	-- draw returns coords if it wants to be drawn as dot
			TI(dots,ret)
		end
	end
	if #dots > 0 then
		gl.PointSize(5)
		gl.VertexPointer(dots)
		gl.EnableClientState('VERTEX_ARRAY')
		gl.DrawArrays('POINTS',0,#dots)
		gl.DisableClientState('VERTEX_ARRAY')
	end
	
	
	-- beacons
	for i,v in ipairs(self.beacons) do
		v:draw()
	end

	-- boats
	
	gl.Enable('LIGHTING')
	
	for i,v in ipairs(self.boats) do
		v:draw()
	end
	
	for i,v in ipairs(self.teles) do
		v:draw()
	end
	
	for i,v in ipairs(self.shelters) do
		v:draw()
	end
	
	gl.Disable('LIGHTING')
	
	-- cursor

	gl.Color(1,0,0)
	gl.PointSize(5)
	gl.Begin('POINTS')
		gl.Vertex(self.camx, self.camy,self.camalt + 0.1)
	gl.End('POINTS')

	gl.PopMatrix()
		
	-- menu
	if gMenu then
		gMenu:draw()
	end
	
	if self.gameover then
		self:gameoverdraw()
	end
	
end

function Game:normalinput(dt)
	
	local addbeacon = function()
		
		if self.gameover then
			return
		end
		
		-- remove one selected
		if game.hot and game.hot:isa(Beacon) then
			for i,v in ipairs(self.beacons) do
				if v == game.hot then
					table.remove(self.beacons,i)
					break
				end
			end
		else
			
			-- or add a new one
			local b = Beacon(self.camx,self.camy)
			if b then
				TI(self.beacons, b)
			end
		end
		
	-- check limit number of beacons
		if #self.beacons > 10 then
			table.remove(self.beacons,1)
		end
		
		for i,v in ipairs(self.dudes) do
			v:checkBeacons()
		end				
		
	end
	
	local addboat = function()
		
		if self.gameover then
			return
		end
			
		local b = Boat(self.camx,self.camy)
		if b then
				TI(self.boats, b)
		end

		
	end
	
	
	local cbtab = {
		fps = function()
			print('fps:', gFps)
		end,
		wireframe = function()
			wireframe = not wireframe
		end,
		
		beacon = addbeacon,
		
		boat = addboat,
		
		action = function()
			
			if self.gameover then
				return
			end
			
			-- let the current targeting object conclude
			if self.targeting then
				self.targeting:target()
				return
			end
			
			-- or select the hot object
			if self.hot then
				self.hot:action()
				return
			end
			
			-- else, big menu
			local cb = {}
			
			TI(cb,{beaconmesh, addbeacon, 0.5})
			
			if #self.boats < 5 then
				TI(cb, {boatmesh, addboat, 0.5})
			end
			
			
			if #self.teles < 4 then
				TI(cb, {telemesh, function()
					local t = Tele(self.camx,self.camy,self.camalt)
					if t then
						TI(self.teles,t)
					end
								
				end, 0.5})
			end
			
			TI(cb,{sheltermesh, function()
				local t = Shelter(self.camx,self.camy,self.camalt)
				if t then
					TI(self.shelters,t)
				end
			
			end,0.25})
			
			gMenu = Menu(cb)
		
		end,
		
		cancel = function()
			
			if self.targeting then
			
				self.targeting:canceltargeting()
				self.targeting = nil
				return
			end
		end,
		
		quit = function()
			
			quit = true
		end,
		
		color = function()
			
			landcol = landcol % 2 + 1
		end
	}
	
	evol.treatAllInput(gMap,cbtab)
	
	
	-- Camera movement !
	
	local maxspeed = lerp(6,60,(self.camz-2)/60)
	local speedx,speedy = 0,0
	
	if evol.vState'up' == 'down' then
		speedy = maxspeed
	elseif evol.vState'down' == 'down' then
		speedy = -maxspeed
	end
	if evol.vState'left' == 'down' then
		speedx = -maxspeed
	elseif evol.vState'right' == 'down' then
		speedx = maxspeed
	end
	
	local rots = 160
	if evol.vState'rotleft' == 'down' then
		self.camdir = self.camdir - rots*dt
	elseif evol.vState'rotright' == 'down' then
		self.camdir = self.camdir + rots*dt
	end
	
	dirx = sin(rad(self.camdir))
	diry = cos(rad(self.camdir))
	
	vx = speedy * dirx + speedx * diry
	vy = speedy * diry - speedx * dirx
	
	self.camx = self.camx + vx*dt
	self.camy = self.camy + vy*dt
	
	if self.camx > self.size-0.1 then self.camx = self.size-0.1 end
	if self.camy > self.size-0.1 then self.camy = self.size-0.1 end
	if self.camx < 0.1 then self.camx = 0.1 end
	if self.camy < 0.1 then self.camy = 0.1 end
	
	-- and follow land
	self.camalt = self:altitude(self.camx,self.camy) or self.camalt
	
	-- finally, smooth movement
	local smoothspeed = 10
	
	self.camx2 = self.camx2 + (self.camx - self.camx2) * smoothspeed * dt
	self.camy2 = self.camy2 + (self.camy - self.camy2) * smoothspeed * dt
	self.camalt2 = self.camalt2 + (self.camalt - self.camalt2) * smoothspeed * dt
	
	-- zoom
	
	local zspeed = 50
	if evol.vState'zoomin' == 'down' then
		self.camz = self.camz - zspeed*dt
	elseif evol.vState'zoomout' == 'down' then
		self.camz = self.camz + zspeed*dt
	end
	
	if self.camz < 3 then self.camz = 3 end
	if self.camz > maxcamz then self.camz = maxcamz end
	
end

function Game:update(dt)
		
	local cbtab = {}
	
	if gMenu then
		gMenu:input(dt)
	
	else
		self:normalinput(dt)
	end
		
	
	-- light
	self.light = self.light + 1/60*360*dt
	
	-- sea level
	self.sealevel = self.sealevel + 1.5/60*dt
	if evol.vState('turbo') == 'down' then
		self.sealevel = self.sealevel + 1*dt
	end
	
		
	--dudes!
	for i,v in ipairs(self.dudes) do
		v:update(dt)
	end

	
	-- kill dudes
	local i = 1
	while i <= #self.dudes do
		if self.dudes[i].die then
			table.remove(self.dudes,i)
		else
			i = i+1
		end
	end
	
	-- check hotness
	local hotd = 2
	local hotlist = {}
	
	game.hot = nil
	for i,v in ipairs(self.beacons) do
		local d = dist2(v,{x=self.camx,y=self.camy})
		if d < hotd^2 then
			TI(hotlist,{v,d})
		end
	end
	for i,v in ipairs(self.boats) do
		local d = dist2(v,{x=self.camx,y=self.camy})
		if d < hotd^2 then
			TI(hotlist,{v,d})
		end
	end
	for i,v in ipairs(self.teles) do
		local d = dist2(v,{x=self.camx,y=self.camy})
		if d < hotd^2 then
			TI(hotlist,{v,d})
		end
	end
	for i,v in ipairs(self.shelters) do
		local d = dist2(v,{x=self.camx,y=self.camy})
		if d < hotd^2 then
			TI(hotlist,{v,d})
		end
	end
	
	if #hotlist > 0 then
		table.sort(hotlist, function(a,b) return a[2] < b[2] end)
		self.hot = hotlist[1][1]
	end
	
	
	-- beacons
	local i = 1
	while i <= #self.beacons do
		if self.beacons[i].die then
			table.remove(self.beacons,i)
		else
			i = i+1
		end
	end
	
	-- boats
	for i,v in ipairs(self.boats) do
		v:update(dt)
	end
	
	local i = 1
	while i <= #self.boats do
		if self.boats[i].die then
			table.remove(self.boats,i)
		else
			i = i+1
		end
	end
	
	-- teles
	
	for i,v in ipairs(self.teles) do
		v:update(dt)
	end
	
	local i = 1
	while i <= #self.teles do
		if self.teles[i].die then
			table.remove(self.teles,i)
		else
			i = i+1
		end
	end
	
	-- shelters
	
	for i,v in ipairs(self.shelters) do
		v:update(dt)
	end
	
	local i = 1
	while i <= #self.shelters do
		if self.shelters[i].die then
			table.remove(self.shelters,i)
		else
			i = i+1
		end
	end
	
	-- count saved
	local total = 0
		
	-- check boats
	for i,v in ipairs(self.boats) do
		total = total + v.load
	end	
	
	-- check shelters
	for i,v in ipairs(self.shelters) do
		total = total + v.load
	end	
	self.saved = total
	
	-- aaand ending condition!	
	if #self.dudes == 0 then
		
		self.gameover = true
	end
	
	
end

function Game:altitude(x,y)
	
	return math.max(self.land:altitude(x,y) or 0, self.sealevel)
	
end

function Game:gameoverupdate(dt)
	
	evol.treatAllInput(gMap,{
		quit = function()
			
			quit = true
		end		
	})
end

function Game:gameoverdraw()
		
	gl.MatrixMode('MODELVIEW')
	gl.LoadIdentity()
	
	gl.LineWidth(2)
	gl.Color(1,1,1)
	
	gl.PushMatrix()
		gl.Translate(0,0,-4)		
		gl.Scale(0.3,0.3,0.3)
		drawNumber(game.saved)
	gl.PopMatrix()
	
end
