-- Copyright (C) 2010 Noé Falzon

-- This program is  free software:  you can redistribute it and/or modify it
-- under  the terms  of the  GNU General Public License  as published by the
-- Free Software Foundation, version 3 of the License.
--
-- This program  is  distributed  in the hope  that it will  be useful,  but
-- WITHOUT  ANY WARRANTY;  without  even the implied  warranty of MERCHANTA-
-- BILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the  GNU General Public License  along
-- with this program. If not, see <http://www.gnu.org/licenses/>.

require "class"

Tele = class()

local top = 1.892

function Tele:init(x,y,z)
	
	self.x = x
	self.y = y
	self.z = z
	
	self.link = nil
	
	if not telelinkmesh then
		telelinkmesh = obj.load("meshes/telelink.obj")
		obj.makeDisplayList(telelinkmesh, false, false)
	end
	
	if not telecallmesh then
		telecallmesh = obj.load("meshes/telecall.obj")
		obj.makeDisplayList(telecallmesh, false, false)
	end
	
	-- check for water
	if z <= game.sealevel then
		return
	end
	
	self.building = 100	-- percents
	
	return self
end

function Tele:draw()
	
	if game.hot == self then
		gl.Color(col.hot)
	else
		gl.Color(col.buildings)
	end
	
	-- main mesh
	
	if self.building > 0 then
		gl.Enable('CLIP_PLANE0')
	end
	
	gl.Enable('LIGHTING')
	
	gl.PushMatrix()
	gl.Translate(self.x,self.y,self.z)
	gl.Rotate(90,1,0,0)
	
	gl.ClipPlane('CLIP_PLANE0',{0,-1,0,top*(100-self.building)/100})
	
	obj.drawDisplayList(telemesh)
	
	gl.Disable('CLIP_PLANE0')
	
	gl.PopMatrix()
	
	
	-- lightnings
	
	if self.linktargeting then
		if self:checkLOS() then
			self:drawRandLine(game.camx,game.camy,game.camalt)
		else
			gl.Disable('LIGHTING')
			gl.LineWidth(3)
			gl.Color(1,0,0)	
			gl.Begin('LINE')
			gl.Vertex(self.x,self.y,self.z+top)
			gl.Vertex(game.camx,game.camy,game.camalt+top)
			gl.End('LINE')
			gl.Enable('LIGHTING')
		end
	end
	
	if self.link then
		
		self:drawRandLine(self.link.x,self.link.y,self.link.z)
	end
	
end


function Tele:target()
	
	if self.linktargeting then
	
		-- check presence of other tele
		if game.hot and game.hot:isa(Tele) and self:checkLOS() and game.hot.building <= 0 then
			
			self.link = game.hot
			
			self.linktargeting = false
			game.targeting = nil
		end
		
	end
	
end

function Tele:drawRandLine(x,y,z)

	local step = 5
	
	gl.Disable('LIGHTING')
	gl.LineWidth(3)
	
	for i = 1,5 do
		gl.Begin('LINE_STRIP')
		
		local ra = MR()
		gl.Color(1-ra,1-ra,1)
		
		gl.Vertex(self.x,self.y,self.z+top)
		for i = 1/step,1,1/step do
			gl.Vertex(lerp(self.x,x,i)+MR2()*step, lerp(self.y,y,i)+MR2()*step, lerp(self.z+top,z,i)+MR2()*step)
		end
		gl.Vertex(x,y,z+top)
		
		gl.End('LINE_STRIP')
	end
	
	gl.Enable('LIGHTING')
end

function Tele:canceltargeting()

	if self.linktargeting then
	
		self.linktargeting = nil
		game.targeting = nil
	
	end
end

function Tele:checkLOS()
	
	local x,y,z = game.camx,game.camy,game.camalt
	
	local step = 10
	
	for i = 0/step,1,1/step do
		local x2,y2,z2 = lerp(self.x,x,i), lerp(self.y,y,i), lerp(self.z+top,z+top,i)
		if z2 <= game:altitude(x2,y2) then
			return false
		end
	end
	
	return true
end

function Tele:update(dt)
	
	if self.z <= game.sealevel then
		self.die = true
	end
	
	-- build
	if self.building > 0 then
		self.building = self.building - 2*dt
	end
	
	-- check if linked didn't die
	if self.link and self.link.die then
		self.link = nil
	end
	
end

function Tele:action()
	
	local cb = {}
	
	if self.building <= 0 then
		
		TI(cb,{telelinkmesh, function()
				
			game.targeting = self
			self.linktargeting = true
			
		end})
		
		if self.link then
			TI(cb,{telecallmesh, function()
				
				for i,v in ipairs(game.dudes) do
				
					if dist2(self,v) < 5^2 then
						v:target(self.x,self.y)
					end

				end
				
				
			end})
		end
	
	end
		
	TI(cb,{deletemesh, function()
			
		self.die = true
	end})
	
	gMenu = Menu(cb)

end

function Tele:teleport(dude)
	
	if not self.link then
		return
	end
	
	dude.x = self.link.x + MR2()*2
	dude.y = self.link.y + MR2()*2
	
end
