require "utils"
require "glutils"

local function make_mesh(v,f)
	
	local l = {}
	
	for _,face in ipairs(f) do
		
		for _,vert in ipairs(face) do
			
			TI(l, v[vert][1])			
			TI(l, v[vert][2])
			TI(l, v[vert][3])
			
		end
	end

	return glutils.makeArray(l)
end

Mesh = class()

function Mesh:init(v,f)
	
	self.v = v
	self.f = f
	
	self.varray = make_mesh(v,f)
	
	self.n = {}
	self:makeNormals()
	
	return self
end

function Mesh:makeNormals()
	
	for i,f in ipairs(self.f) do
		self.n[i] = cross_prod_norm(self.v[f[1]],self.v[f[2]],self.v[f[3]])
	end
	
	local t = {}
	for i,n in ipairs(self.n) do
		
		TI(t,n[1])
		TI(t,n[2])
		TI(t,n[3])
		
		TI(t,n[1])
		TI(t,n[2])
		TI(t,n[3])
		
		TI(t,n[1])
		TI(t,n[2])
		TI(t,n[3])
	end
	
	self.narray = glutils.makeArray(t)
	
end

function Mesh:draw()
	
	gl.EnableClientState('VERTEX_ARRAY')
	gl.EnableClientState('NORMAL_ARRAY')
	glutils.setVertexPointer(self.varray)
	glutils.setNormalPointer(self.narray)
	gl.DrawArrays('TRIANGLES',0,3*#self.f)
	gl.DisableClientState('VERTEX_ARRAY')
	gl.DisableClientState('VERTEX_ARRAY')
end
