#include <GL/gl.h>
#include <GL/glu.h>

#include "lua.h"
#include "lualib.h"
#include "lauxlib.h"

struct vArray
{
	int n;
	double a[1];
};

static int l_makeArray (lua_State *L)
{	
	struct vArray *array;
	int n,i;
	
	if (!lua_istable(L,-1))
	{
		luaL_error(L,"Argument must be a table");
		return 0;
	}
	
	n = lua_objlen(L,-1);
	
	array = lua_newuserdata(L, n*sizeof(double)+sizeof(int));
	array->n = n;
	
	
	for(i=0; i < n; i++)
	{
		lua_pushnumber(L,i+1);
		lua_gettable(L,-3);
		if( !lua_isnumber(L,-1) )
		{
			luaL_error(L,"Table must contain numbers only");
			return 0;
		}
		array->a[i] = lua_tonumber(L,-1);
		lua_pop(L,1);
	}
	
	/* userdata is still on top, return it */
		
	return 1;
}

static int l_setVertexPointer (lua_State *L)
{
	struct vArray *array;
	
	array = lua_touserdata(L,1);
	
	glVertexPointer(3,GL_DOUBLE,0,array->a);
	
	return 0;
}

static int l_setNormalPointer (lua_State *L)
{
	struct vArray *array;
	
	array = lua_touserdata(L,1);
	
	glNormalPointer(GL_DOUBLE,0,array->a);
	
	return 0;
}

/* Making this a Lua module */

static const struct luaL_reg R[] = {
	{"makeArray", l_makeArray},
	{"setVertexPointer", l_setVertexPointer},
	{"setNormalPointer", l_setNormalPointer},
	
	{NULL, NULL}
};

int luaopen_glutils (lua_State *L)
{
	luaL_register(L, "glutils", R);
	return 1;
}
