-- Copyright (C) 2010 Noé Falzon

-- This program is  free software:  you can redistribute it and/or modify it
-- under  the terms  of the  GNU General Public License  as published by the
-- Free Software Foundation, version 3 of the License.
--
-- This program  is  distributed  in the hope  that it will  be useful,  but
-- WITHOUT  ANY WARRANTY;  without  even the implied  warranty of MERCHANTA-
-- BILITY  or  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public
-- License for more details.
--
-- You should have received a copy of the  GNU General Public License  along
-- with this program. If not, see <http://www.gnu.org/licenses/>.

-- Some shortcuts and utilities

MR = math.random
MR2 = function()
	return MR()-0.5
end

MF = math.floor
TI = table.insert

sin = math.sin
cos = math.cos
rad = math.rad


PP = function(t)

	if not t then print 'nil' return end

	if type(t) == 'number' then
		io.write(t,"'")

	elseif type(t) ~= 'table' then
		io.write(tostring(t))
	else

		io.write'('
		for i,v in ipairs(t) do
			PP(v)

			if i < #t then io.write',' end
		end
		io.write')'

	end

end

-- normalised cross prod of 23 and 21
function cross_prod_norm(v1,v2,v3)


	local u = { v3[1] - v2[1], v3[2] - v2[2], v3[3] - v2[3] }	-- 3 - 2
	local v = { v1[1] - v2[1], v1[2] - v2[2], v1[3] - v2[3] }	-- 1 - 2
	
	-- u x v
	local n = {
		u[2]*v[3] - u[3]*v[2],
		u[3]*v[1] - u[1]*v[3],
		u[1]*v[2] - u[2]*v[1]
	}
	
	-- normalize
	local norm = math.sqrt(n[1]^2 + n[2]^2 + n[3]^2)
	n[1] = n[1]/norm
	n[2] = n[2]/norm
	n[3] = n[3]/norm
	
	return n
end


function seed_perlin(n)

	local tab = {}
	for i=0,n do
		tab[i] = {}
		for j = 0,n do
			tab[i][j] = MR()
		end
	end
	
	gPerlin = tab
end

function read_perlin(x,y,wl)
	
	local i,j = MF(x/wl),MF(y/wl)
	
	local px,py = x/wl-i,y/wl-j
	
	local t1 = sinerp(gPerlin[i][j],gPerlin[i+1][j],px)
	local t2 = sinerp(gPerlin[i][j+1],gPerlin[i+1][j+1],px)
	
	return sinerp(t1,t2,py)
end

function sinerp(a,b,x)

	return a + (math.cos(math.pi*(1-x)) * 0.5 + 0.5)*(b-a)
end

function lerp(a,b,x)
	return a + x*(b-a)
end

function dist2(a,b)
	return (a.x-b.x)^2 + (a.y-b.y)^2
end


---[[ Vector numbers ! ]]
-- takes list of segments
function vert7seg(t2)
	
	local t = {}
	for i,v in ipairs(t2) do
		t[v] = true
	end
	
	gl.Begin('LINES')
	
	if t[1] then
		gl.Vertex(0,0)
		gl.Vertex(1,0)
	end
	
	if t[2] then
		gl.Vertex(0,0)
		gl.Vertex(0,1)
	end
	
	if t[3] then
		gl.Vertex(1,0)
		gl.Vertex(1,1)
	end
	
	if t[4] then
		gl.Vertex(0,1)
		gl.Vertex(1,1)
	end
	
	if t[5] then
		gl.Vertex(0,1)
		gl.Vertex(0,2)
	end
	
	if t[6] then
		gl.Vertex(1,1)
		gl.Vertex(1,2)
	end
	
	if t[7] then
		gl.Vertex(0,2)
		gl.Vertex(1,2)
	end
	
	gl.End('LINES')
end

-- n 0-9
function vertNum(n)
	
	local tab = {
		[0] = {1,2,3,5,6,7},
		{3,6},
		{1,2,4,6,7},
		{1,3,4,6,7},
		{3,4,5,6},
		{1,3,4,5,7},
		{1,2,3,4,5,7},
		{3,6,7},
		{1,2,3,4,5,6,7},
		{1,3,4,5,6,7}
	}
	
	vert7seg(tab[n])
end

-- any n
function drawNumber(n)
	
	local digits = {}
	local n = n
	
	while n > 0 do
		TI(digits,1,n%10)
		n = MF(n/10)
	end
	
	if #digits == 0 then
		digits = {0}
	end

	for i,v in ipairs(digits) do
		vertNum(v)
		gl.Translate(1.2,0,0)
	end
end


function C(t)
	
	return {t[1]/255,t[2]/255,t[3]/255}
	
end
